import unittest
from urlchecker_db import LogDB
import peewee
from urlchecker_dns import expandDomainList, logDnsQuery
from urlchecker_status import logStatus


class UrlCheckerDNSTestCase(unittest.TestCase):
    def setUp(self):
        self.db = LogDB(":memory:")
        self.db.createTables()

    ################################################
    # expandDomainList
    ################################################
    def test_expandDomainList(self):
        result = expandDomainList(["c", "b.a.a", "a.a.a", "a.a"])
        assert result == ["a.a", "a.a.a", "b.a.a", "c"]

    ################################################
    # logDnsQuery
    ################################################
    def test_logDnsQuery_insertFirst(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = 'foo'
        answer_list = ['4.3.2.1', '1.2.3.4']
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == '1.2.3.4, 4.3.2.1'
        assert self.db.DnsChange.get().status_id == status_id
        assert result == status_id

    def test_logDnsQuery_insertOnlyOnePerStatusIdIPUrl(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = 'foo'
        answer_list = ['4.3.2.1', '1.2.3.4']
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        answer_list.extend('127.0.0.1')
        try:
            logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        except peewee.IntegrityError:
            assert self.db.DnsChange.select().count() == 1
            assert self.db.DnsChange.get().status_id == status_id
        else:
            raise NotImplementedError("Expected IntegrityError")

    def test_logDnsQuery_skipIdenticalPreviousValues(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = 'foo'
        answer_list = ['4.3.2.1', '1.2.3.4']
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        status_id_2 = logStatus(self.db, "foo")
        result_2 = logDnsQuery(self.db, status_id_2, resolver_ip, domain, rdtype, answer_list)

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == '1.2.3.4, 4.3.2.1'
        assert self.db.DnsChange.get().status_id == status_id
        assert result == status_id
        assert result == result_2

    def test_logDnsQuery_insertWhenDifferentAnswer(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = 'foo'
        answer_list = ['4.3.2.1', '1.2.3.4']
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        answer_list_2 = ['4.3.2.1', '1.2.3.4', '0.0.0.0']
        status_id_2 = logStatus(self.db, "foo")
        result_2 = logDnsQuery(self.db, status_id_2, resolver_ip, domain, rdtype, answer_list_2)

        assert result_2 != result
        assert self.db.DnsChange.select().count() == 2

        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id).resolver_ip == resolver_ip
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id).domain == domain
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id).rdtype == rdtype
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id).response == '1.2.3.4, 4.3.2.1'
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id).status_id == status_id

        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id_2).resolver_ip == resolver_ip
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id_2).domain == domain
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id_2).rdtype == rdtype
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id_2).response == '0.0.0.0, 1.2.3.4, 4.3.2.1'
        assert self.db.DnsChange.get(self.db.DnsChange.status == status_id_2).status_id == status_id_2

    def test_logDnsQuery_insertDifferentUrl(self):
        domain = "http://example.org"
        domain_2 = domain + '.'
        resolver_ip = "127.0.0.1"
        resolver_ip_2 = resolver_ip + '1'
        rdtype = 'foo'
        rdtype_2 = rdtype + 'bar'
        answer_list = ['4.3.2.1', '1.2.3.4']
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip_2, domain, rdtype, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip, domain_2, rdtype, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip, domain, rdtype_2, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip_2, domain_2, rdtype, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip_2, domain, rdtype_2, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip, domain_2, rdtype_2, answer_list)
        logDnsQuery(self.db, status_id, resolver_ip_2, domain_2, rdtype_2, answer_list)

        assert self.db.DnsChange.select().count() == 8


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(UrlCheckerDNSTestCase))
    return suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")
